﻿/*
 * CREDITS: this example is based on material developed for the
 * Introduction to Programming course by Nimi Berman and Cay Horstmann
 * at San Jose State University, CA.
 * 
 * The original material can be found at:
 *    www.cs.sjsu.edu/faculty/horstman/CS46A/debugger/tutorial.html
 */

using System;
using System.Linq;

namespace WordAnalyzer
{
  /// <summary>
  /// A class that analyzes words.
  /// </summary>
  public class WordAnalyzer
  {
    private String _word = "";

    /// <summary>
    /// Setter method for field word.
    /// </summary>
    /// <param name="word">the word to be analyzed</param>
    /// <exception cref="ArgumentException">Thrown when <paramref name="word"/> is equal to null.</exception>
    public void SetWord(String word)
    {
      if (word != null)
      {
        _word = word;
      }
      else
      {
        throw new ArgumentException("The word needs to be non-null.");
      }
    }

    /// <summary>
    /// Gets the first repeated character. A character is <i>repeated</i>
    /// if it occurs at least twice in adjacent positions.
    /// </summary>
    /// <example>
    /// For example, 'l' is repeated in "hollow", but 'o' is not.
    /// </example>
    /// <returns>the first repeated character, or 0 if none found</returns>
    public char FirstRepeatedCharacter()
    {
      for (int i = 0; i < _word.Length; i++)
      {
        char ch = _word.ElementAt(i);
        if (i + 1 < _word.Length && ch == _word.ElementAt(i + 1))
        {
          return ch;
        }
      }
      return (char) 0;
    }

    /// <summary>
    /// Gets the first multiply occuring character. A character is <i>multiple</i>
    /// if it occurs at least twice in the word, not necessarily in adjacent positions.
    /// </summary>
    /// <example>
    /// For example, both 'o' and 'l' are multiple in "hollow", but 'h' is not.
    /// </example>
    /// <returns>the first multiple character, or 0 if none found</returns>
    public char FirstMultipleCharacter()
    {
      for (int i = 0; i < _word.Length; i++)
      {
        char ch = _word.ElementAt(i);
        if (Find(ch, i) >= 0)
        {
          return ch;
        }
      }
      return (char) 0;
    }

    private int Find(char c, int pos)
    {
      for (int i = pos + 1; i < _word.Length; i++)
      {
        if (_word.ElementAt(i) == c)
        {
          return i;
        }
      }
      return -1;
    }

    /// <summary>
    /// Counts the groups of repeated characters.
    /// </summary>
    /// <example>
    /// For example, "mississippi!!!" has four such groups: ss, ss, pp and !!!.
    /// </example>
    /// <returns>the number of repeated character groups</returns>
    public int CountRepeatedCharacters()
    {
      int c = 0;

      for (int i = 0; i < _word.Length - 1; i++)
      {
        if (_word.ElementAt(i) == _word.ElementAt(i + 1))
        {
          // found a repetition
          if (i == 0 || _word.ElementAt(i - 1) != _word.ElementAt(i)) // it's the start
          {
            c++;
          }
        }
      }
      return c;
    }

    /// <summary>
    /// Is the word a palindrome? A palindrome is a word that is equal to its
    /// reversed representation.
    /// </summary>
    /// <example>
    /// For example, "abba" is a palindrome.
    /// </example>
    public bool IsPalindrome()
    {
      for (int i = 0; i < (_word.Length - i - 1); i++)
      {
        if (_word.ElementAt(i) != _word.ElementAt(_word.Length - i - 1))
        {
          return false;
        }
      }
      return true;
    }
  }
}