﻿/*
 * CREDITS: this example is based on material developed for the
 * Introduction to Programming course by Nimi Berman and Cay Horstmann
 * at San Jose State University, CA.
 *
 * The original material can be found at:
 *    www.cs.sjsu.edu/faculty/horstman/CS46A/debugger/tutorial.html
 */


using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Moles;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WordAnalyzer.Tests
{
  /// <summary>
  /// Summary description for TestWordAnalyzer
  /// </summary>
  [TestClass]
  public class TestWordAnalyzer
  {
    // Test fixture
    private WordAnalyzer _wa;

    /// <summary>
    /// Gets or sets the test context which provides
    /// information about and functionality for the current test run.
    /// </summary>
    public TestContext TestContext { get; set; }

    #region Additional test attributes

    // You can use the following additional attributes as you write your tests:

    // Use ClassInitialize to run code before running the first test in the class
    [ClassInitialize]
    public static void MyClassInitialize(TestContext testContext)
    {
    }

    // Use ClassCleanup to run code after all tests in a class have run
    [ClassCleanup]
    public static void MyClassCleanup()
    {
    }

    // Use TestInitialize to run code before running each test 
    [TestInitialize]
    public void MyTestInitialize()
    {
      _wa = new WordAnalyzer();
    }

    // Use TestCleanup to run code after each test has run
    [TestCleanup]
    public void MyTestCleanup()
    {
    }

    #endregion

    // Test of firstRepeatedCharacter method, of class WordAnalyzer.
    [TestMethod]
    public void TestFirstRepeatedCharacter()
    {
      _wa.SetWord("");
      Assert.AreEqual(0, _wa.FirstRepeatedCharacter());

      _wa.SetWord("aardvark");
      Assert.AreEqual('a', _wa.FirstRepeatedCharacter());

      _wa.SetWord("roommate");
      Assert.AreEqual('o', _wa.FirstRepeatedCharacter());

      _wa.SetWord("mate");
      Assert.AreEqual(0, _wa.FirstRepeatedCharacter());

      _wa.SetWord("test");
      Assert.AreEqual(0, _wa.FirstRepeatedCharacter());
    }

    // Test of firstMultipleCharacter method, of class WordAnalyzer.
    [TestMethod]
    public void TestFirstMultipleCharacter()
    {
      _wa.SetWord("");
      Assert.AreEqual(0, _wa.FirstMultipleCharacter());

      _wa.SetWord("missisippi");
      Assert.AreEqual('i', _wa.FirstMultipleCharacter());

      _wa.SetWord("mate");
      Assert.AreEqual(0, _wa.FirstMultipleCharacter());

      _wa.SetWord("test");
      Assert.AreEqual('t', _wa.FirstMultipleCharacter());
    }


    // Test of countRepeatedCharacters method, of class WordAnalyzer.
    [TestMethod]
    public void TestCountRepeatedCharacters()
    {
      _wa.SetWord("");
      Assert.AreEqual(0, _wa.CountRepeatedCharacters());

      _wa.SetWord("mississippiii");
      Assert.AreEqual(4, _wa.CountRepeatedCharacters());

      _wa.SetWord("test");
      Assert.AreEqual(0, _wa.CountRepeatedCharacters());

      _wa.SetWord("aabbcdaaaabb");
      Assert.AreEqual(4, _wa.CountRepeatedCharacters());
    }

    // Test of setWord method, of class WordAnalyzer.
    [TestMethod]
    [ExpectedException(typeof (ArgumentException))]
    public void TestSetWordException()
    {
      _wa.SetWord(null);
    }

    // Test of isPalindrome method, of class WordAnalyzer.
    [TestMethod]
    public void TestIsPalindrome()
    {
      _wa.SetWord("");
      Assert.IsTrue(_wa.IsPalindrome());

      _wa.SetWord("a");
      Assert.IsTrue(_wa.IsPalindrome());

      _wa.SetWord("ab");
      Assert.IsFalse(_wa.IsPalindrome());

      _wa.SetWord("aa");
      Assert.IsTrue(_wa.IsPalindrome());

      _wa.SetWord("baab");
      Assert.IsTrue(_wa.IsPalindrome());

      _wa.SetWord("bacab");
      Assert.IsTrue(_wa.IsPalindrome());

      _wa.SetWord("bacdab");
      Assert.IsFalse(_wa.IsPalindrome());
    }

    // Test 'StorePalindrome' using stubs (for interfaces, abstract classes, or virtual methods in non-sealed types).
    [TestMethod]
    public void TestStorePalindromeWithStubs()
    {
      var palindromes = new List<string>();
      StreamWriter s = new SStreamWriter("palindromes.txt")
                         {
                           WriteLineString = (line =>
                                                {
                                                  Assert.IsTrue(line.SequenceEqual(line.Reverse()));
                                                  palindromes.Add(line);
                                                })
                         };

      _wa.SetWord("");
      _wa.StorePalindrome(s);

      _wa.SetWord("ab");
      _wa.StorePalindrome(s);

      _wa.SetWord("abba");
      _wa.StorePalindrome(s);

      _wa.SetWord("abbac");
      _wa.StorePalindrome(s);

      Assert.AreEqual(2, palindromes.Count());
    }

    // Test 'StorePalindrome' using moles (for static methods, sealed types, inline constructor calls).
    [TestMethod]
    [HostType("Moles")]
    public void TestStorePalindromeWithMoles()
    {
      var palindromes = new List<string>();
      const string filename = "palindromes.txt";
      MFile.WriteAllLinesStringStringArray = (path, strings) =>
                                               {
                                                 Assert.IsTrue(strings.All(line => line.SequenceEqual(line.Reverse())));
                                                 palindromes.AddRange(strings);
                                               };

      _wa.SetWord("");
      _wa.StorePalindrome(filename);

      _wa.SetWord("ab");
      _wa.StorePalindrome(filename);

      _wa.SetWord("abba");
      _wa.StorePalindrome(filename);

      _wa.SetWord("abbac");
      _wa.StorePalindrome(filename);

      Assert.AreEqual(2, palindromes.Count());
    }
  }
}